/*
 * HMI.c
 *
 * Created: 27.11.2016 11:52:33
 *  Author: Thomas
 */ 

#include <avr/io.h>
#include "../BasicDefines.h"

#include "HMI.h"
#include "../main.h"
#include "../Analog/Analog.h"
#include "../HSV2RGB/HSV2RGB.h"
#include "../Lib/QuadratureDecoder/QuadratureDecoder.h"


extern uint8_t AnalogPotiHSV_H;
extern uint8_t AnalogPotiHSV_S;
extern uint8_t AnalogPotiHSV_V;

extern uint8_t ClockPulse100ms;
extern uint8_t AutomaticColorChangePulse;


uint8_t HMI_AdjustmentHSV_H;
uint8_t HMI_AdjustmentHSV_S;
uint8_t HMI_AdjustmentHSV_V;

uint8_t SelectedMode;


typedef enum tagModeSelectStateMachine {
	DarkIdle,
	WakeUp,
	SelectNewMode,
} ModeSelectStateMachine_T, *pModeSelectStateMachine_T;


static ModeSelectStateMachine_T ModeSelectStateMachine;
static QuadratureDec_T QD_ModeSelection;
static uint8_t ModeLEDs;
static uint8_t PreModeLEDs;


void HMI_Init()
{
	DDRC |= (0x0f<<DDC2);	// LED Outputs

	PORTD |= (7<<PORTD5);	// Encoder/switch pull up resistors on
	QuadratureDecoderInit(((PIND & (3<<PIND5)) >> 5), &QD_ModeSelection);
	
	ModeLEDs = 1;
	
	ModeSelectStateMachine = SelectNewMode;
	
	AnalogInit();
}


void HMI_Main()
{
	static int8_t PreModeCounter;
	static uint8_t DelayWakeUpCounter;
	static uint8_t DelayDarkIdleCounter;
	static uint8_t DelaySelectCounter;
	

	AnalogReadChannels();

	#ifndef TEST_BOARD
		HMI_AdjustmentHSV_H = AnalogPotiHSV_H;
		HMI_AdjustmentHSV_S = (255-(256>>2)) + ((AnalogPotiHSV_S+1)>>2);
		HMI_AdjustmentHSV_V = AnalogPotiHSV_V;
	#else
		HMI_AdjustmentHSV_H = ((255UL * 30) / 360);
		HMI_AdjustmentHSV_S = 255;
		HMI_AdjustmentHSV_V = 255;
	#endif
	
	
	int8_t tmpPreModeCounter = PreModeCounter;
	
	QuadratureEncoding(((PIND & (3<<PIND5)) >> 5), &QD_ModeSelection);
	int8_t tmpSteps = QuadratureDecoderDiv4Step(&QD_ModeSelection);
	

	switch (ModeSelectStateMachine)
	{
		case DarkIdle:
		{
			if (tmpSteps)
			{
				ModeSelectStateMachine = WakeUp;
			}
		}
		break;


		case WakeUp:
		{
			if (ClockPulse100ms)
			{
				uint8_t tmpDelayWakeUpCounter = DelayWakeUpCounter;
	
				if (++tmpDelayWakeUpCounter >= DELAY_WAKE_UP)
				{
					tmpDelayWakeUpCounter = 0;
	
					DelaySelectCounter = 0;
					DelayDarkIdleCounter = 0;

					ModeSelectStateMachine = SelectNewMode;
				}
				
				DelayWakeUpCounter = tmpDelayWakeUpCounter;
			}
		}
		break;


		case SelectNewMode:
		{
			tmpPreModeCounter += tmpSteps;
			
			if (tmpSteps)
			{
				DelaySelectCounter = 0;
				DelayDarkIdleCounter = 0;
			}
			else if (ClockPulse100ms)
			{
				uint8_t tmpDelaySelectCounter = DelaySelectCounter;
				uint8_t tmpDelayDarkIdleCounter = DelayDarkIdleCounter;
				
				if (++tmpDelaySelectCounter >= DELAY_SELECT_MODE)
				{
					tmpDelaySelectCounter = 0;
					
					SelectedMode = PreModeCounter;
					
					ModeLEDs = PreModeLEDs;
				}
				
				if (++tmpDelayDarkIdleCounter >= DELAY_DARK_IDLE)
				{
					ModeSelectStateMachine = DarkIdle;
				}
				
				DelaySelectCounter = tmpDelaySelectCounter;
				DelayDarkIdleCounter = tmpDelayDarkIdleCounter;
			}
		}
		break;
	}
	
	
	// Limit PreModeCounter
	if (tmpPreModeCounter < 0)
	{
		tmpPreModeCounter = 0;
	}
	else if (tmpPreModeCounter > (MODE_COUNT-1))
	{
		tmpPreModeCounter = (MODE_COUNT-1);
	}

	PreModeLEDs = (1<<(uint8_t)tmpPreModeCounter);
	PreModeCounter = tmpPreModeCounter;
	
	
	// Status LEDs handling
	static uint8_t FlashingLEDs;
	
	if (ClockPulse100ms)
	{
		FlashingLEDs ^= 0xff;
	}
	
	uint8_t tmpLEDPort = 0;
	
	if (ModeSelectStateMachine != DarkIdle)
	{
		if (ModeLEDs != (1<<(MODE_COUNT-1)))
		{
			if (PreModeLEDs != (1<<(MODE_COUNT-1)))
			{
				tmpLEDPort = ModeLEDs | (FlashingLEDs & PreModeLEDs);
			}
			else
			{
				tmpLEDPort = ModeLEDs | (FlashingLEDs & 0x0f);
			}
		}
		else
		{
			tmpLEDPort = ~(FlashingLEDs & PreModeLEDs);
		}
	}

	PORTC = (PORTC & ~(0x0f<<PORTC2)) | ((tmpLEDPort & 0x0f)<<PORTC2);
	
	
	// Encoder Switch handling
	static uint8_t EncoderSwitch;
	static uint8_t EdgeEncoderSwitch;
	
	if (ClockPulse100ms)
	{
		EncoderSwitch = ~PIND & 0x80;
	}
		
	if (EncoderSwitch)
	{
		if (!EdgeEncoderSwitch)
		{
			AutomaticColorChangePulse = 0xff;
		}
	}
	EdgeEncoderSwitch = EncoderSwitch;
}
